"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var fsextra = __importStar(require("fs-extra"));
var path = __importStar(require("path"));
var generateOutputFilename = function (extractTo, 
// In test env then opts.file isn't available
filename) {
    if (filename === void 0) { filename = "test.jsx"; }
    var basename = path.basename(filename, path.extname(filename));
    // Make sure the relative path is "absolute" before
    // joining it with the `messagesDir`.
    var relativePath = path.join(path.sep, path.relative(process.cwd(), filename));
    // Solve when the window user has symlink on the directory, because
    // process.cwd on windows returns the symlink root,
    // and filename (from babel) returns the original root
    if (process.platform === "win32") {
        var name_1 = path.parse(process.cwd()).name;
        if (relativePath.includes(name_1)) {
            relativePath = relativePath.slice(relativePath.indexOf(name_1) + name_1.length);
        }
    }
    return path.join(extractTo, path.dirname(relativePath), basename + ".json");
};
var TEST_IDS = Symbol("TEST_IDS");
var DEFAULT_MAGIC_OBJECT = "$TestId";
function plugin(_a) {
    var t = _a.types;
    return {
        /**
         * Create a Set on the file to hold collected ids. Attaching it to the file
         * means there is no issue of shared mutable state across different files
         */
        pre: function (file) {
            if (!file.has(TEST_IDS)) {
                file.set(TEST_IDS, new Set());
            }
        },
        visitor: {
            /**
             *
             * Prevent referring to the naked magicObject identifier, e.g.
             * { x: $TestId }
             */
            Identifier: function (path) {
                var _a = this.opts.magicObject, magicObject = _a === void 0 ? DEFAULT_MAGIC_OBJECT : _a;
                if (path.node.name === magicObject &&
                    !t.isMemberExpression(path.container)) {
                    throw path.buildCodeFrameError("Cannot refer to '" + magicObject + "' as a literal value. Use dot access to produce testIds e.g. '" + magicObject + ".myTestId'");
                }
            },
            MemberExpression: function (path, state) {
                var _a = this.opts.magicObject, magicObject = _a === void 0 ? DEFAULT_MAGIC_OBJECT : _a;
                if (t.isIdentifier(path.node.object) &&
                    path.node.object.name === magicObject &&
                    t.isIdentifier(path.node.property)) {
                    // The testid is the name of that property
                    var testId = path.node.property.name;
                    // Get existing ids, and as long as there's no dupe, add this one
                    var ids = state.file.get(TEST_IDS);
                    if (ids.has(testId)) {
                        throw path.buildCodeFrameError("Duplicate test id: " + testId + "}");
                    }
                    ids.add(testId);
                    path.replaceWith(t.stringLiteral(testId));
                }
            }
        },
        /**
         * Once we've traversed the file, if we have found any ids then write
         * them to a json file, with a path that corresponds to the original
         * location of the file, but relative the the `testIdsDir`
         */
        post: function (file) {
            var _a = this.opts || {
                extractTo: false
            }, extractTo = _a.extractTo, _b = _a.fs, fs = _b === void 0 ? fsextra : _b;
            if (!extractTo) {
                return;
            }
            var ids = file.get(TEST_IDS);
            var foundKeys = ids.size;
            if (foundKeys > 0) {
                var output = JSON.stringify(__spread(ids.values()), null, 2);
                var filename = file.opts.filename;
                var idFileName = generateOutputFilename(extractTo, filename);
                fs.mkdirpSync(path.dirname(idFileName));
                fs.writeFileSync(idFileName, output);
            }
        }
    };
}
exports.plugin = plugin;
exports.default = plugin;
